@@ -1,7 +1,7 @@
 /* challenge.c: The opiechallenge() library function.
 
-%%% portions-copyright-cmetz
-Portions of this software are Copyright 1996 by Craig Metz, All Rights
+%%% portions-copyright-cmetz-96
+Portions of this software are Copyright 1996-1998 by Craig Metz, All Rights
 Reserved. The Inner Net License Version 2 applies to these portions of
 the software.
 You should have received a copy of the license with this software. If
@@ -14,6 +14,8 @@ License Agreement applies to this software.
 
         History:
 
+	Modified by cmetz for OPIE 2.32. Added extended response set
+		identifier to the challenge.
 	Modified by cmetz for OPIE 2.3. Use opie_ prefix. Send debug info to
 		syslog. Add sha plumbing.
 	Modified by cmetz for OPIE 2.2. Use FUNCTION declaration et al.
@@ -68,7 +70,7 @@ int opiechallenge FUNCTION((mp, name, ss), struct opie *mp AND char *name AND ch
     opierandomchallenge(ss);
     memset(mp, 0, sizeof(*mp));
   } else
-    sprintf(ss, "otp-%s %d %s", algids[MDX], mp->opie_n - 1, mp->opie_seed);
+    sprintf(ss, "otp-%s %d %s ext", algids[MDX], mp->opie_n - 1, mp->opie_seed);
 
   return rval;
 }
