@@ -1,8 +1,11 @@
 /* readpass.c: The opiereadpass() library function.
 
-Portions of this software are Copyright 1996 by Craig Metz, All Rights
-Reserved. The Inner Net Copyright Notice and License Agreement applies to
-these portions of the software.
+%%% portions-copyright-cmetz-96
+Portions of this software are Copyright 1996-1998 by Craig Metz, All Rights
+Reserved. The Inner Net License Version 2 applies to these portions of
+the software.
+You should have received a copy of the license with this software. If
+you didn't get a copy, you may request one from <license@inner.net>.
 
 Portions of this software are Copyright 1995 by Randall Atkinson and Dan
 McDonald, All Rights Reserved. All Rights under this copyright are assigned
@@ -11,6 +14,10 @@ License Agreement applies to this software.
 
         History:
 
+	Modified by cmetz for OPIE 2.31. Use usleep() to delay after setting
+		the terminal attributes; this might help certain buggy
+		systems.
+	Modified by cmetz for OPIE 2.3. Use TCSAFLUSH always.
 	Modified by cmetz for OPIE 2.22. Replaced echo w/ flags.
                Really use FUNCTION.
 	Modified by cmetz for OPIE 2.2. Use FUNCTION declaration et al.
@@ -61,9 +68,6 @@ static jmp_buf jmpbuf;
 
 static VOIDRET catch FUNCTION((i), int i)
 {
-#if DEBUG
-  fprintf(stderr, "Got to catcher!\n");
-#endif /* DEBUG */
   longjmp(jmpbuf, 1);
 }
 #endif /* unix */
@@ -193,8 +197,16 @@ char *opiereadpass FUNCTION((buf, len, flags), char *buf AND int len AND int fla
     }
 #endif /* VEOF */
 
-    if (tcsetattr(0, (flags & 2) ? TCSANOW : TCSAFLUSH, &attr))
+#if HAVE_USLEEP
+    usleep(1);
+#endif /* HAVE_USLEEP */
+
+    if (tcsetattr(0, TCSAFLUSH, &attr))
       goto error;
+
+#if HAVE_USLEEP
+    usleep(1);
+#endif /* HAVE_USLEEP */
   }
 #else /* unix */
   erase[0] = CONTROL('H');
@@ -278,7 +290,7 @@ restore:
 #ifdef unix
   /* Restore previous tty modes */
   if (isatty(0))
-    if (tcsetattr(0, TCSANOW, &orig_attr))
+    if (tcsetattr(0, TCSAFLUSH, &orig_attr))
       return NULL;
 
   signal(SIGINT, SIG_DFL);
