@@ -1,8 +1,11 @@
 /* opielogin.c: The infamous /bin/login
 
-Portions of this software are Copyright 1996 by Craig Metz, All Rights
-Reserved. The Inner Net Copyright Notice and License Agreement applies to
-these portions of the software.
+%%% portions-copyright-cmetz-96
+Portions of this software are Copyright 1996-1998 by Craig Metz, All Rights
+Reserved. The Inner Net License Version 2 applies to these portions of
+the software.
+You should have received a copy of the license with this software. If
+you didn't get a copy, you may request one from <license@inner.net>.
 
 Portions of this software are Copyright 1995 by Randall Atkinson and Dan
 McDonald, All Rights Reserved. All Rights under this copyright are assigned
@@ -11,6 +14,21 @@ License Agreement applies to this software.
 
 	History:
 
+	Modified by cmetz for OPIE 2.32. Partially handle environment
+		variables on the command line (a better implementation is
+		coming soon). Handle failure to issue a challenge more
+		gracefully.
+	Modified by cmetz for OPIE 2.31. Use _PATH_NOLOGIN. Move Solaris
+	        drain bamage kluge after rflag check; it breaks rlogin.
+		Use TCSAFLUSH instead of TCSANOW (except where it flushes
+		data we need). Sleep before kluging for Solaris.
+	Modified by cmetz for OPIE 2.3. Process login environment files.
+	        Made logindevperm/fbtab handling more generic. Kluge around
+                Solaris drain bamage differently (maybe better?). Maybe
+		allow cleartext logins even when opiechallenge() fails.
+		Changed the conditions on when time.h and sys/time.h are
+		included. Send debug info to syslog. Use opielogin() instead
+		of dealing with utmp/setlogin() here.
 	Modified by cmetz for OPIE 2.22. Call setlogin(). Decreased default
 	        timeout to two minutes. Use opiereadpass() flags to get
 		around Solaris drain bamage.
@@ -72,7 +90,11 @@ License Agreement applies to this software.
 #endif /* HAVE_SYS_PARAM_H */
 #include <sys/stat.h>
 #include <sys/types.h>
+
 #if HAVE_SETPRIORITY && HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif /* HAVE_SETPRIORITY && HAVE_SYS_RESOURCE_H */
+
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
 # include <time.h>
@@ -83,18 +105,10 @@ License Agreement applies to this software.
 #include <time.h>
 #endif /* HAVE_SYS_TIME_H */
 #endif /* TIME_WITH_SYS_TIME */
-#include <sys/resource.h>
-#else /* HAVE_SETPRIORITY && HAVE_SYS_RESOURCE_H */
-#if TM_IN_SYS_TIME
-#include <sys/time.h>
-#else /* TM_IN_SYS_TIME */
-#include <time.h>
-#endif /* TM_IN_SYS_TIME */
-#endif /* HAVE_SETPRIORITY && HAVE_SYS_RESOURCE_H */
+
 #if HAVE_SYS_FILE_H
 #include <sys/file.h>
 #endif /* HAVE_SYS_FILE_H */
-#include <utmp.h>
 #include <signal.h>
 #if HAVE_PWD_H
 #include <pwd.h>	/* POSIX Password routines */
@@ -117,13 +131,14 @@ License Agreement applies to this software.
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
+#if HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif /* HAVE_SYS_SELECT_H */
 
 #ifdef	QUOTA
 #include <sys/quota.h>
 #endif
 
-struct utmp utmp;
-
 #if HAVE_GETTTYNAM
 #include <sys/ioctl.h>	/* non-portable routines used only a few places */
 #include <ttyent.h>
@@ -131,24 +146,10 @@ struct utmp utmp;
 
 #include "opie.h"
 
-#ifndef DODISC	/* Is the line discipline code even necessary? */
-#define DODISC 0
-#endif	/* DODISC */
-
-#if DOUTMPX
-/* On many System V systems, we include and use extended utmp -- utmpx
-because the ut_host member is present in utmpx but not utmp. The defines in
-opie_cfg.h should also help with portability. */
-
-#include <utmpx.h>
-struct utmpx utmpx;
-
-#endif	/* DOUTMPX */
-
 #define TTYGID(gid)	tty_gid(gid)	/* gid that owns all ttys */
 
-#define NMAX	sizeof(UTMPX.ut_name)
-#define HMAX	sizeof(UTMPX.ut_host)
+#define NMAX	32
+#define HMAX	256
 
 #if HAVE_LASTLOG_H
 #include <lastlog.h>
@@ -159,11 +160,10 @@ static int usererr = -1;
 static int stopmotd;
 static char rusername[NMAX + 1];
 static char name[NMAX + 1] = "";
-/* char me[MAXHOSTNAMELEN]; */
 static char minusnam[16] = "-";
 static char *envinit[1];	/* now set by setenv calls */
 static char term[64] = "\0";	/* important to initialise to a NULL string */
-static char host[sizeof(UTMPX.ut_host)] = "\0";
+static char host[HMAX + 1] = "\0";
 static struct passwd nouser;
 static struct passwd thisuser;
 
@@ -288,7 +288,7 @@ static VOIDRET getloginname FUNCTION_NOARGS
     flags = 1;
     if (ttyprompt) {
       if (first) {
-	flags = 6;
+	flags = 4;
 	first--;
       } else
 	printf(ttyprompt);
@@ -328,7 +328,7 @@ static VOIDRET catch FUNCTION((i), int i)
 static VOIDRET catchexit FUNCTION_NOARGS
 {
   int i;
-  tcsetattr(STDIN_FILENO, TCSANOW, &attr);
+  tcsetattr(STDIN_FILENO, TCSAFLUSH, &attr);
   putchar('\n');
   closelog();
   for (i = sysconf(_SC_OPEN_MAX); i > 2; i--)
@@ -396,13 +396,13 @@ verify that the line being tested isn't commented out check for the substring
     }	/* end while another line could be obtained */
   }	/* end if could open file */
   return (1);	/* when no CONSOLE line exists, root can login from anywhere */
-#elif DOSECURETTY
+#elif HAVE_SECURETTY
   {
     FILE *f;
     char buffer[1024], *c;
     int rc = 0;
 
-    if (!(f = fopen(SECURETTY, "r")))
+    if (!(f = fopen("/etc/securetty", "r")))
       return 1;
 
     if (c = strstr(ttyn, "/dev/"))
@@ -642,12 +642,23 @@ int main FUNCTION((argc, argv), int argc AND char *argv[])
   int i;
   char *p;
   char opieprompt[OPIE_CHALLENGE_MAX + 1];
-  int pwok, af_pwok;
+  int af_pwok;
+  int authsok;
   char *pp;
   char buf[256];
   int uid;
   int opiepassed;
 
+#ifndef DEBUG
+  if (geteuid()) {
+    fprintf(stderr, "This program requires super-user priveleges.\n");
+    exit(1);
+  }
+#endif /* DEBUG */
+
+  for (t = sysconf(_SC_OPEN_MAX); t > 2; t--)
+    close(t);
+
   openlog("login", LOG_ODELAY, LOG_AUTH);
 
   /* initialisation */
@@ -656,7 +667,7 @@ int main FUNCTION((argc, argv), int argc AND char *argv[])
 
   if (p = getenv("TERM")) {
 #ifdef DEBUG
-    fprintf(stderr, "Environment TERM=%s", p);
+    syslog(LOG_DEBUG, "environment TERM=%s", p);
 #endif /* DEBUG */
     strncpy(term, p, sizeof(term));
   };
@@ -688,120 +699,110 @@ int main FUNCTION((argc, argv), int argc AND char *argv[])
   {
     int foo;
 
-    printf("my args are: (argc=%d)\n", foo = argc);
+    syslog(LOG_DEBUG, "my args are: (argc=%d)", foo = argc);
     while (--foo)
-      printf("%d: %s\n", foo, argv[foo]);
+      syslog(LOG_DEBUG, "%d: %s", foo, argv[foo]);
   }
-#endif	/* DEBUG */
+#endif /* DEBUG */
 
-/* Some OSs pass environment variables on the command line. All of them except
-   for TERM get eaten. */
-
-  i = argc;
-  while (--i)
-    if (strchr(argv[i], '=')) {
-#ifdef DEBUG
-      printf("eating %s\n", argv[i]);
-#endif	/* DEBUG */
-      argc--;
-      if (!strncmp(argv[i], "TERM=", 5)) {
-	strncpy(term, &(argv[i][5]), sizeof(term));
-	term[sizeof(term) - 1] = 0;
-#ifdef DEBUG
-	printf("passed TERM=%s, ouroptind = %d\n", term, i);
-#endif	/* DEBUG */
-      }
-    }
 /* Implement our own getopt()-like functionality, but do so in a much more
    strict manner to prevent security problems. */
   for (ouroptind = 1; ouroptind < argc; ouroptind++) {
-    i = 0;
-    if (argv[ouroptind])
-      if (argv[ouroptind][0] == '-')
-	if (i = argv[ouroptind][1])
-	  if (!argv[ouroptind][2])
-	    switch (i) {
-	    case 'd':
-	      if (++ouroptind == argc)
-		exit(1);
+    if (!argv[ouroptind])
+      continue;
+
+    if (argv[ouroptind][0] == '-') {
+      char *c = argv[ouroptind] + 1;
+
+      while(*c) {
+	switch(*(c++)) {
+	  case 'd':
+	    if (*c || (++ouroptind == argc))
+	      exit(1);
+
 /*    The '-d' option is apparently a performance hack to get around
    ttyname() being slow. The potential does exist for it to be used
    for malice, and it does not seem to be strictly necessary, so we
    will just eat it. */
-	      break;
+	    break;
 
-	    case 'r':
+	  case 'r':
+	    if (rflag || hflag || fflag) {
+	      fprintf(stderr, "Other options not allowed with -r\n");
+	      exit(1);
+	    }
+
+	    if (*c || (++ouroptind == argc))
+	      exit(1);
+
+	    if (!(ouroptarg = argv[ouroptind]))
+	      exit(1);
+
+	    rflag = -1;
+	    if (!doremotelogin(ouroptarg))
+	      rflag = 1;
+	    
+	    strncpy(host, ouroptarg, sizeof(host));
+	    break;
+
+	  case 'h':
+	    if (!getuid()) {
 	      if (rflag || hflag || fflag) {
-		printf("Other options not allowed with -r\n");
+		fprintf(stderr, "Other options not allowed with -h\n");
 		exit(1);
 	      }
-	      if (++ouroptind == argc)
+	      hflag = 1;
+
+	      if (*c || (++ouroptind == argc))
 		exit(1);
 
-	      ouroptarg = argv[ouroptind];
-
-	      if (!ouroptarg)
+	      if (!(ouroptarg = argv[ouroptind]))
 		exit(1);
-
-	      rflag = -1;
-	      if (!doremotelogin(ouroptarg))
-		rflag = 1;
-
+	      
 	      strncpy(host, ouroptarg, sizeof(host));
-	      break;
+	    }
+	    break;
 
-	    case 'h':
-	      if (!getuid()) {
-		if (rflag || hflag || fflag) {
-		  printf("Other options not allowed with -h\n");
-		  exit(1);
-		}
-		hflag = 1;
+	  case 'f':
+	    if (rflag) {
+	      fprintf(stderr, "Only one of -r and -f allowed\n");
+	      exit(1);
+	    }
+	    fflag = 1;
 
-		if (++ouroptind == argc)
-		  exit(1);
+	    if (*c || (++ouroptind == argc))
+	      exit(1);
 
-		ouroptarg = argv[ouroptind];
+	    if (!(ouroptarg = argv[ouroptind]))
+	      exit(1);
 
-		if (!ouroptarg)
-		  exit(1);
-		
-		strncpy(host, ouroptarg, sizeof(host));
-	      }
-	      break;
+	    strncpy(name, ouroptarg, sizeof(name));
+	    break;
+	  case 'p':
+	    pflag = 1;
+	    break;
+	};
+      };
+      continue;
+    };
 
-	    case 'f':
-	      if (rflag) {
-		printf("Only one of -r and -f allowed\n");
-		exit(1);
-	      }
-	      fflag = 1;
+    if (strchr(argv[ouroptind], '=')) {
+      if (!strncmp(argv[ouroptind], "TERM=", 5)) {
+	strncpy(term, &(argv[ouroptind][5]), sizeof(term));
+	term[sizeof(term) - 1] = 0;
+#ifdef DEBUG
+	syslog(LOG_DEBUG, "passed TERM=%s, ouroptind = %d", term, ouroptind);
+#endif /* DEBUG */
+      } else {
+#ifdef DEBUG
+	syslog(LOG_DEBUG, "eating %s, ouroptind = %d", argv[ouroptind], ouroptind);
+#endif /* DEBUG */
+      };
+      continue;
+    };
 
-	      if (++ouroptind == argc)
-		exit(1);
-
-	      ouroptarg = argv[ouroptind];
-
-	      if (!ouroptarg)
-		exit(1);
-
-	      strncpy(name, ouroptarg, sizeof(name));
-	      break;
-
-	    case 'p':
-	      pflag = 1;
-	      break;
-	  } else
-	    i = 0;
-    if (!i) {
-      ouroptarg = argv[ouroptind++];
-      strncpy(name, ouroptarg, sizeof(name));
-      break;
-    }
-  }
-
-  for (t = sysconf(_SC_OPEN_MAX); t > 2; t--)
-    close(t);
+    strncpy(name, argv[ouroptind], sizeof(name));
+  };
 
 #ifdef TIOCNXCL
   /* BSDism:  not sure how to rewrite for POSIX.  rja */
@@ -816,6 +817,49 @@ int main FUNCTION((argc, argv), int argc AND char *argv[])
    across the network. */
   if (rflag)
     doremoteterm(term);
+  else {
+    struct termios termios;
+    fd_set fds;
+    struct timeval timeval;
+    
+    memset(&timeval, 0, sizeof(struct timeval));
+    
+    FD_ZERO(&fds);
+    FD_SET(0, &fds);
+
+#if HAVE_USLEEP
+    usleep(1);
+#endif /* HAVE_USLEEP */
+
+    if (select(1, &fds, NULL, NULL, &timeval)) {
+#ifdef DEBUG
+      syslog(LOG_DEBUG, "reading user name from tty buffer");
+#endif /* DEBUG */
+
+      if (tcgetattr(0, &termios)) {
+#ifdef DEBUG
+	syslog(LOG_DEBUG, "tcgetattr(0, &termios) failed");
+#endif /* DEBUG */
+	exit(1);
+      }
+      
+      termios.c_lflag &= ~ECHO;
+   
+      if (tcsetattr(0, TCSANOW, &termios)) {
+#ifdef DEBUG
+	syslog(LOG_DEBUG, "tcsetattr(0, &termios) failed");
+#endif /* DEBUG */
+	exit(1);
+      }
+
+      if ((i = read(0, name, sizeof(name)-1)) > 0)
+	name[i] = 0;
+      if ((p = strchr(name, '\r')))
+        *p = 0;
+      if ((p = strchr(name, '\n')))
+        *p = 0;
+    }
+  }
 
 /* Force termios portable control characters to the system default values as
 specified in termios.h. This should help the one-time password login feel the
@@ -943,7 +987,7 @@ completeness, but these are set within appropriate defines for portability. */
   attr.c_cflag |= HUPCL;	/* hangup on close */
 
   /* Set revised termio attributes */
-  if (tcsetattr(STDIN_FILENO, TCSANOW, &attr))
+  if (tcsetattr(STDIN_FILENO, TCSAFLUSH, &attr))
     return (-1);
 
   atexit(catchexit);
@@ -964,32 +1008,46 @@ completeness, but these are set within appropriate defines for portability. */
 #endif /* HAVE_SETVBUF && defined(_IONBF) */
 
 #ifdef DEBUG
-  fprintf(stderr, "tty = %s\n", tty);
-#endif	/* DEBUG */
+  syslog(LOG_DEBUG, "tty = %s", tty);
+#endif /* DEBUG */
 
-#ifdef hpux
+#ifdef HAVE_LOGIN_ENVFILE
   {
     FILE *f;
 
-    if (f = fopen("/etc/src.sh", "r")) {
-      char line[80], *c;
+    if (f = fopen(HAVE_LOGIN_ENVFILE, "r")) {
+      char line[128], *c, *c2;
 
-      while(fgets(line, sizeof(line)-1, f))
-        if (!strncmp(line, "TZ=", 3) && (c = strchr(line, ';'))) {
-           *c = 0;
-           putenv(line);
+      while(fgets(line, sizeof(line)-1, f)) {
+	c = line;
+	while(*c && (isalnum(*c) || (*c == '_'))) c++;
+	  if (*c == '=') {
+	    *(c++) = 0;
+	    if (c2 = strchr(c, ';'))
+	      *c2 = 0;
+	    if (c2 = strchr(c, '\n'))
+	      *c2 = 0;
+	    if (c2 = strchr(c, ' '))
+	      continue;
+	    if (c2 = strchr(c, '\t'))
+	      continue;
+	    if (!strcmp(line, "TZ"))
+	      continue;
+	    if (setenv(line, c, 1) < 0) {
+	      fprintf(stderr, "setenv() failed -- environment full?\n");
+	      break;
+	    }
+	  }
         }
       fclose(f);
     }
   }
-#endif /* hpux */
+#endif /* HAVE_LOGIN_ENVFILE */
 
   t = 0;
   invalid = TRUE;
   af_pwok = opieaccessfile(host);
 
-  opiegetutmpentry(tty, &UTMPX);
-
   if (name[0])
     if (name[0] == '-') {
       fprintf(stderr, "User names can't start with '-'.\n");
@@ -1003,11 +1061,11 @@ completeness, but these are set within appropriate defines for portability. */
     if (invalid && !name[0]) {
       getloginname();
       invalid = lookupuser();
+      authsok = 0;
     }
 #ifdef DEBUG
-    printf("login name is -%s-, of length %d, [0] = %d\n", name,
-	   strlen(name), name[0]);
-#endif	/* DEBUG */
+    syslog(LOG_DEBUG, "login name is +%s+, of length %d, [0] = %d", name, strlen(name), name[0]);
+#endif /* DEBUG */
 
     if (fflag) {
       uid = getuid();
@@ -1025,12 +1083,18 @@ completeness, but these are set within appropriate defines for portability. */
        prompt for and verify a password. */
     if (!fflag && (rflag < 1) && *thisuser.pw_passwd) {
 #ifdef DEBUG
-      printf("login name is -%s-, of length %d, [0] = %d\n", name,
-	     strlen(name), name[0]);
-#endif	/* DEBUG */
+      syslog(LOG_DEBUG, "login name is +%s+, of length %d, [0] = %d\n", name, strlen(name), name[0]);
+#endif /* DEBUG */
 
       /* Attempt a one-time password challenge */
-      opiechallenge(&opie, name, opieprompt);
+      i = opiechallenge(&opie, name, opieprompt);
+
+      if ((i < 0) || (i > 1)) {
+        syslog(LOG_ERR, "error: opiechallenge() returned %d, errno=%d!\n", i, errno);
+      } else {
+        printf("%s\n", opieprompt);
+	authsok |= 1;
+      }
 
       if (!memcmp(&thisuser, &nouser, sizeof(thisuser)))
 	if (host[0])
@@ -1040,21 +1104,29 @@ completeness, but these are set within appropriate defines for portability. */
 	  syslog(LOG_WARNING, "Invalid login attempt for %s on %s.",
 		 name, tty);
 
-      pwok = af_pwok && opiealways(thisuser.pw_dir);
+      if (af_pwok && opiealways(thisuser.pw_dir))
+	authsok |= 2;
+
 #if DEBUG
-      fprintf(stderr, "af_pwok = %d, pwok = %d\n", af_pwok, pwok);
+      syslog(LOG_DEBUG, "af_pwok = %d, authsok = %d", af_pwok, authsok);
 #endif /* DEBUG */
 
-      printf("%s\n", opieprompt);
+      if (!authsok)
+	syslog(LOG_ERR, "no authentication methods are available for %s!", name);
+
 #if NEW_PROMPTS
-      if (pwok)
-        printf("Response or Password: ");
-      else
-        printf("Response: ");
-      if (!opiereadpass(buf, sizeof(buf), !pwok))
+      if ((authsok & 1) || !authsok)
+        printf("Response");
+      if (((authsok & 3) == 3) || !authsok)
+        printf(" or ");
+      if ((authsok & 2) || !authsok)
+        printf("Password");
+      printf(": ");
+      fflush(stdout);
+      if (!opiereadpass(buf, sizeof(buf), !(authsok & 2)))
         invalid = TRUE;
 #else /* NEW_PROMPTS */
-      if (!pwok)
+      if (!(authsok & 1) && authsok)
 	printf("(OTP response required)\n");
       printf("Password:");
       fflush(stdout);
@@ -1062,8 +1134,8 @@ completeness, but these are set within appropriate defines for portability. */
         invalid = TRUE;
 #endif /* NEW_PROMPTS */
 
-      if (!buf[0]) {
-        pwok = 0;
+      if (!buf[0] && (authsok & 1)) {
+        authsok &= ~2;
 	/* Null line entered, so display appropriate prompt & flush current
 	   data. */
 #if NEW_PROMPTS
@@ -1075,39 +1147,36 @@ completeness, but these are set within appropriate defines for portability. */
           invalid = TRUE;
       }
 
-/* Once opieverify() is called, this information is gone */
-      i = opiegetsequence(&opie);
-
-      opiepassed = !opieverify(&opie, buf);
+      if (authsok & 1) {
+        i = opiegetsequence(&opie);
+        opiepassed = !opieverify(&opie, buf);
 
 #ifdef DEBUG
-      fprintf(stderr, "opiepassed = %d\n", opiepassed);
+      syslog(LOG_DEBUG, "opiepassed = %d", opiepassed);
 #endif /* DEBUG */
+      }
 
       if (!invalid) {
-        if (opiepassed) {
-	    /* OPIE authentication succeeded */
-	    if (i < 5) {
-	      printf("Warning: Change your OTP secret pass phrase NOW!\n");
-	    } else {
-	      if (i < 10) {
-		printf("Warning: Change your OTP secret pass phrase.\n");
-	      }
-            }
+        if ((authsok & 1) && opiepassed) {
+	  if (i < 10) {
+	    printf("Warning: Re-initialize your OTP information");
+            if (i < 5)
+              printf(" NOW!");
+            printf("\n");
+	  }
         } else {
-	    /* Try regular password check, if allowed */
-	    if (pwok) {
-	      pp = crypt(buf, thisuser.pw_passwd);
-	      invalid = strcmp(pp, thisuser.pw_passwd);
-	    } else
-              invalid = TRUE;
+	  if (authsok & 2) {
+	    pp = crypt(buf, thisuser.pw_passwd);
+	    invalid = strcmp(pp, thisuser.pw_passwd);
+	  } else
+            invalid = TRUE;
 	}
       }
     }
 
     /* If user not super-user, check for logins disabled. */
     if (thisuser.pw_uid) {
-      if (nlfd = fopen(NO_LOGINS_FILE, "r")) {
+      if (nlfd = fopen(_PATH_NOLOGIN, "r")) {
 	while ((c = getc(nlfd)) != EOF)
 	  putchar(c);
 	fflush(stdout);
@@ -1176,16 +1245,8 @@ completeness, but these are set within appropriate defines for portability. */
   }
 #endif
 
-  strncpy(UTMPX.ut_host, host, sizeof(UTMPX.ut_host));
-  strncpy(UTMPX.ut_name, name, sizeof(UTMPX.ut_name));
-#ifdef USER_PROCESS
-  UTMPX.ut_type = USER_PROCESS;
-#endif	/* USER_PROCESS */
-
-  opiepututmpentry(tty, &UTMPX);
-#if HAVE_SETLOGIN
-  setlogin(name);
-#endif /* HAVE_SETLOGIN */
+  if (opielogin(tty, name, host))
+    syslog(LOG_ERR, "can't record login: tty %s, name %s, host %s", tty, name, host);
 
   quietlog = !access(QUIET_LOGIN_FILE, F_OK);
 
@@ -1278,10 +1339,42 @@ interested in hearing of a more portable approach. rja */
   setenv("PATH", DEFAULT_PATH, 0);
   if (term[0]) {
 #ifdef DEBUG
-    printf("setting TERM=%s\n", term);
+    syslog(LOG_DEBUG, "setting TERM=%s", term);
 #endif	/* DEBUG */
     setenv("TERM", term, 1);
   }
+
+#ifdef HAVE_LOGIN_ENVFILE
+  {
+    FILE *f;
+
+    if (f = fopen(HAVE_LOGIN_ENVFILE, "r")) {
+      char line[128], *c, *c2;
+
+      while(fgets(line, sizeof(line)-1, f)) {
+	c = line;
+	while(*c && (isalnum(*c) || (*c == '_'))) c++;
+	  if (*c == '=') {
+	    *(c++) = 0;
+	    if (c2 = strchr(c, ';'))
+	      *c2 = 0;
+	    if (c2 = strchr(c, '\n'))
+	      *c2 = 0;
+	    if (c2 = strchr(c, ' '))
+	      continue;
+	    if (c2 = strchr(c, '\t'))
+	      continue;
+	    if (setenv(line, c, 0) < 0) {
+	      fprintf(stderr, "setenv() failed -- environment full?\n");
+	      break;
+	    }
+	  }
+        }
+      fclose(f);
+    }
+  }
+#endif /* HAVE_LOGIN_ENVFILE */
+
   if ((namep = strrchr(thisuser.pw_shell, '/')) == NULL)
     namep = thisuser.pw_shell;
   else
@@ -1325,7 +1418,7 @@ interested in hearing of a more portable approach. rja */
 
     strcat(buf, name);
 #if DEBUG
-    fprintf(stderr, "statting %s\n", buf);
+    syslog(LOG_DEBUG, "statting %s", buf);
 #endif /* DEBUG */
     if (!stat(buf, &st) && st.st_size)
       printf("You have %smail.\n",
